package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMDRWG;
import cn.ibizlab.eam.core.eam_core.filter.EMDRWGSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMDRWGService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMDRWGMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[文档] 服务对象接口实现
 */
@Slf4j
@Service("EMDRWGServiceImpl")
public class EMDRWGServiceImpl extends ServiceImpl<EMDRWGMapper, EMDRWG> implements IEMDRWGService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMDRWGMapService emdrwgmapService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEmpService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EMDRWG et) {
        fillParentData(et);
        emobjectService.update(emdrwgInheritMapping.toEmobject(et));
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emdrwgid",et.getEmdrwgid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmdrwgid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMDRWG> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public EMDRWG getDraft(EMDRWG et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean save(EMDRWG et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMDRWG et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EMDRWG> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EMDRWG> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMDRWG et) {
        return (!ObjectUtils.isEmpty(et.getEmdrwgid()))&&(!Objects.isNull(this.getById(et.getEmdrwgid())));
    }
    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        emobjectService.remove(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean create(EMDRWG et) {
        fillParentData(et);
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmdrwgid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMDRWG> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMDRWG get(String key) {
        EMDRWG et = getById(key);
        if(et==null){
            et=new EMDRWG();
            et.setEmdrwgid(key);
        }
        else{
        }
        return et;
    }


	@Override
    public List<EMDRWG> selectByBpersonid(String pfempid) {
        return baseMapper.selectByBpersonid(pfempid);
    }

    @Override
    public void removeByBpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMDRWG>().eq("bpersonid",pfempid));
    }

	@Override
    public List<EMDRWG> selectByRempid(String pfempid) {
        return baseMapper.selectByRempid(pfempid);
    }

    @Override
    public void removeByRempid(String pfempid) {
        this.remove(new QueryWrapper<EMDRWG>().eq("rempid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMDRWG> searchDefault(EMDRWGSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMDRWG> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMDRWG>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMDRWG et){
        //实体关系[DER1N_EMDRWG_PFEMP_BPERSONID]
        if(!ObjectUtils.isEmpty(et.getBpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp bperson=et.getBperson();
            if(ObjectUtils.isEmpty(bperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getBpersonid());
                et.setBperson(majorEntity);
                bperson=majorEntity;
            }
            et.setBpersonname(bperson.getPfempname());
        }
        //实体关系[DER1N_EMDRWG_PFEMP_REMPID]
        if(!ObjectUtils.isEmpty(et.getRempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp remp=et.getRemp();
            if(ObjectUtils.isEmpty(remp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getRempid());
                et.setRemp(majorEntity);
                remp=majorEntity;
            }
            et.setRempname(remp.getPfempname());
        }
    }



    @Autowired
    cn.ibizlab.eam.core.eam_core.mapping.EMDRWGInheritMapping emdrwgInheritMapping;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMObjectService emobjectService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(EMDRWG et){
        if(ObjectUtils.isEmpty(et.getEmdrwgid()))
            et.setEmdrwgid((String)et.getDefaultKey(true));
        cn.ibizlab.eam.core.eam_core.domain.EMObject emobject =emdrwgInheritMapping.toEmobject(et);
        emobject.set("emobjecttype","DRWG");
        emobjectService.create(emobject);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EMDRWG> getEmdrwgByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EMDRWG> getEmdrwgByEntities(List<EMDRWG> entities) {
        List ids =new ArrayList();
        for(EMDRWG entity : entities){
            Serializable id=entity.getEmdrwgid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



