package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMWork;
import cn.ibizlab.eam.core.eam_core.filter.EMWorkSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMWorkService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMWorkMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[加班工单] 服务对象接口实现
 */
@Slf4j
@Service("EMWorkServiceImpl")
public class EMWorkServiceImpl extends ServiceImpl<EMWorkMapper, EMWork> implements IEMWorkService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEmpService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EMWork et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmworkid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMWork> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMWork et) {
        return (!ObjectUtils.isEmpty(et.getEmworkid()))&&(!Objects.isNull(this.getById(et.getEmworkid())));
    }
    @Override
    @Transactional
    public EMWork get(String key) {
        EMWork et = getById(key);
        if(et==null){
            et=new EMWork();
            et.setEmworkid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EMWork getDraft(EMWork et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean update(EMWork et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emworkid",et.getEmworkid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmworkid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMWork> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMWork et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMWork et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EMWork> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EMWork> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }


	@Override
    public List<EMWork> selectByEmpid(String pfempid) {
        return baseMapper.selectByEmpid(pfempid);
    }

    @Override
    public void removeByEmpid(String pfempid) {
        this.remove(new QueryWrapper<EMWork>().eq("empid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMWork> searchDefault(EMWorkSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMWork> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMWork>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMWork et){
        //实体关系[DER1N_EMWORK_PFEMP_EMPID]
        if(!ObjectUtils.isEmpty(et.getEmpid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp emp=et.getEmp();
            if(ObjectUtils.isEmpty(emp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getEmpid());
                et.setEmp(majorEntity);
                emp=majorEntity;
            }
            et.setTeamname(emp.getMajorteamname());
            et.setEmpname(emp.getPfempname());
            et.setDeptid(emp.getMajordeptid());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EMWork> getEmworkByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EMWork> getEmworkByEntities(List<EMWork> entities) {
        List ids =new ArrayList();
        for(EMWork entity : entities){
            Serializable id=entity.getEmworkid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



